package com.engine.organization.web;

import com.engine.common.util.ParamUtil;
import com.engine.common.util.ServiceUtil;
import com.engine.organization.entity.DeleteParam;
import com.engine.organization.entity.postion.param.PostInfoSearchParam;
import com.engine.organization.util.response.ReturnResult;
import com.engine.organization.wrapper.PostInfoWrapper;
import io.swagger.v3.oas.annotations.parameters.RequestBody;
import weaver.hrm.HrmUserVarify;
import weaver.hrm.User;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import java.util.Map;

/**
 * @description:
 * @author:dxfeng
 * @createTime: 2022/05/13
 * @version: 1.0
 */
public class PostInfoController {
    public PostInfoWrapper getPostInfoWrapper(User user) {
        return ServiceUtil.getService(PostInfoWrapper.class, user);
    }

    /**
     * 获取list列表
     *
     * @param request
     * @param response
     * @return
     */
    @GET
    @Path("/getPostInfoTable")
    @Produces(MediaType.APPLICATION_JSON)
    public ReturnResult listPostInfo(@Context HttpServletRequest request, @Context HttpServletResponse response) {
        try {
            User user = HrmUserVarify.getUser(request, response);
            Map<String, Object> map = ParamUtil.request2Map(request);
            return ReturnResult.successed(getPostInfoWrapper(user).listPage(map));
        } catch (Exception e) {
           return ReturnResult.exceptionHandle(e);
        }
    }

    /**
     * 添加职务信息
     *
     * @param request
     * @param response
     * @return
     */
    @POST
    @Path("/savePostInfo")
    @Produces(MediaType.APPLICATION_JSON)
    public ReturnResult savePostInfo(@Context HttpServletRequest request, @Context HttpServletResponse response, @RequestBody PostInfoSearchParam param) {
        try {
            User user = HrmUserVarify.getUser(request, response);
            return ReturnResult.successed(getPostInfoWrapper(user).savePostInfo(param));
        } catch (Exception e) {
           return ReturnResult.exceptionHandle(e);
        }
    }

    /**
     * 更新职务信息，修改所有字段
     *
     * @param request
     * @param response
     * @param param
     * @return
     */
    @POST
    @Path("/updatePostInfo")
    @Produces(MediaType.APPLICATION_JSON)
    public ReturnResult updatePostInfo(@Context HttpServletRequest request, @Context HttpServletResponse response, @RequestBody PostInfoSearchParam param) {
        try {
            User user = HrmUserVarify.getUser(request, response);
            return ReturnResult.successed(getPostInfoWrapper(user).updatePostInfo(param));
        } catch (Exception e) {
           return ReturnResult.exceptionHandle(e);
        }
    }

    /**
     * 更新禁用标记
     *
     * @param request
     * @param response
     * @param param
     * @return
     */
    @POST
    @Path("/updateForbiddenTagById")
    @Produces(MediaType.APPLICATION_JSON)
    public ReturnResult updateForbiddenTagById(@Context HttpServletRequest request, @Context HttpServletResponse response, @RequestBody PostInfoSearchParam param) {
        try {
            User user = HrmUserVarify.getUser(request, response);
            return ReturnResult.successed(getPostInfoWrapper(user).updateForbiddenTagById(param));
        } catch (Exception e) {
           return ReturnResult.exceptionHandle(e);
        }
    }


    /**
     * 根据ID批量删除数据
     *
     * @param request
     * @param response
     * @param param
     * @return
     */
    @POST
    @Path("/deleteByIds")
    @Produces(MediaType.APPLICATION_JSON)
    public ReturnResult deleteByIds(@Context HttpServletRequest request, @Context HttpServletResponse response, @RequestBody DeleteParam param) {
        try {
            User user = HrmUserVarify.getUser(request, response);
            return ReturnResult.successed(getPostInfoWrapper(user).deleteByIds(param.getIds()));
        } catch (Exception e) {
           return ReturnResult.exceptionHandle(e);
        }
    }

    /**
     * 高级搜索条件
     *
     * @param request
     * @param response
     * @return
     */
    @GET
    @Path("/getSearchCondition")
    @Produces(MediaType.APPLICATION_JSON)
    public ReturnResult getSearchCondition(@Context HttpServletRequest request, @Context HttpServletResponse response) {
        try {
            User user = HrmUserVarify.getUser(request, response);
            Map<String, Object> map = ParamUtil.request2Map(request);
            return ReturnResult.successed(getPostInfoWrapper(user).getSearchCondition(map));
        } catch (Exception e) {
           return ReturnResult.exceptionHandle(e);
        }
    }


    /**
     * 新增、编辑表单
     *
     * @param request
     * @param response
     * @return
     */
    @GET
    @Path("/getPostInfoForm")
    @Produces(MediaType.APPLICATION_JSON)
    public ReturnResult getPostInfoForm(@Context HttpServletRequest request, @Context HttpServletResponse response) {
        try {
            User user = HrmUserVarify.getUser(request, response);
            Map<String, Object> map = ParamUtil.request2Map(request);
            return ReturnResult.successed(getPostInfoWrapper(user).getPostInfoForm(map));
        } catch (Exception e) {
           return ReturnResult.exceptionHandle(e);
        }
    }

    /**
     * 列表页顶部按钮
     *
     * @param request
     * @param response
     * @return
     */
    @GET
    @Path("/getHasRight")
    @Produces(MediaType.APPLICATION_JSON)
    public ReturnResult getHasRight(@Context HttpServletRequest request, @Context HttpServletResponse response) {
        try {
            User user = HrmUserVarify.getUser(request, response);
            return ReturnResult.successed(getPostInfoWrapper(user).getHasRight());
        } catch (Exception e) {
           return ReturnResult.exceptionHandle(e);
        }
    }

}
